
// Written by Hans de Nivelle, Dec. 2014.

#ifndef AIRFOIL_MACH0_FULLRANGE_INCLUDED
#define AIRFOIL_MACH0_FULLRANGE_INCLUDED  1

#include <string>
#include <iostream>
#include <cmath>

#include "airfoil_mach0.h"

// airfoil_mach0_fullrange specifies an airfoil at mach 0 over 
// full 360 degrees, from -180 to +180 degrees a.o.a. (but in radians). 

struct airfoil_mach0_fullrange
{

   airfoil_mach0 spec;           // Our specific airfoil.
   airfoil_mach0 def;            // Our default airfoil.

   airfoil_mach0_fullrange( const airfoil_mach0& spec,
                            const airfoil_mach0& def )
      : spec{ spec },
        def{ def } 
   { }

   airfoil_mach0_fullrange( airfoil_mach0&& spec,
                            airfoil_mach0&& def ) noexcept
      : spec{ std::move( spec ) },
        def{ std::move( def ) }
   { }

   // Angle of attack is always in radians!

   // alpha must be between -180.0_deg and +180.0_deg. 
   // We use the specific airfoil where possible, otherwise the
   // default airfoil.

   coefficients getcoefficients( double alpha ) const;
   coefficients operator( ) ( double alpha ) const 
      { return getcoefficients( alpha ); }  
};

#endif


