
#ifndef SCENERY_COLOR_INCLUDED
#define SCENERY_COLOR_INCLUDED   1

#include <iostream>
#include <stdexcept>

namespace scenery
{

   struct color
   {
      float red;
      float green;
      float blue;
 
      // Negative values are rejected, values > 1.0 are quietly accepted. 

      color( float red, float green, float blue )
         : red{ red }, 
           green{ green },
           blue{ blue }
      {
         if( red < 0 || green < 0 || blue < 0 )
            throw std::runtime_error( "color values must be >= 0" );
      } 

      float brightness( ) const
      {
         return sqrt( red * red + green * green + blue * blue );
      }


      // Interpolate between c1 and c2. d = 0 means totally c1.
      // c = 1 means totally c2.
 
      static color interpolate( double d, const color& c1, const color& c2 )
      {
         return color( ( 1 - d ) * c1. red   + d * c2. red,
                       ( 1 - d ) * c1. green + d * c2. green,
                       ( 1 - d ) * c1. blue  + d * c2. blue );
      }

   };


   // A few standard colors: 

   inline color white( )    { return color(1,1,1); } 
   inline color red( )      { return color(1,0,0); }
   inline color green( )    { return color(0,1,0); }
   inline color blue( )     { return color(0,0,1); }
   inline color black( )    { return color(0,0,0); }
   inline color yellow( )   { return color(1,1,0); }
   inline color purple( )   { return color( 0.5, 0, 0.5 ); }
   inline color magenta( )  { return color( 1, 0, 1 ); } 
   inline color orange( )   { return color( 1, 0.6, 0 ); }
   inline color grey( )     { return color( 0.5, 0.5, 0.5 ); }

   inline std::ostream& operator << ( std::ostream& stream, color c )
   {
      stream << "color( ";
      stream << c. red << ", " << c. green << ", " << c. blue; 
      stream << " )";
      return stream;
   }
}

#endif

