
#ifndef GRAPHICS_INCLUDED 
#define GRAPHICS_INCLUDED   1

#include <functional>

#include "linalg/fourmatrix.h"
#include "linalg/affinemap.h"
#include "linalg/rototrans.h"
#include "color.h"


namespace graphics
{

   // Load the perspective into PROJECTION, and the inverse of 
   // position into MODELVIEW. The default is to place ourselves
   // without rotation on Z = 1. 

   void setview( const linalg::fourmatrix& perspective,
                 const linalg::rototrans& position = 
                 linalg::rototrans( {1,0,0,0}, {0,0,1} ) ); 


   // Set light number 0: If inf == true, the light will be
   // in infinity in the direction of position, otherwise 
   // exactly at the given position. 

   void setlight0( linalg::vector lightposition, bool inf, 
                   scenery::color ambient, scenery::color diffuse,
                   scenery::color specular );

   void setfog( float density, scenery::color col );


   // Prepare the current window for 3D-viewing. We clear the screen,
   // enable lighting, etc. 

   void preparefor3d( scenery::color skycolor ); 
  
   // Prepare the current window for 2D-plotting. We clear the screen
   // into black, switch off lighting, fog, etc.
 
   void preparefor2d( ); 

   // Draw a symmetric grid, with the axes in the 
   // middle:  

   void drawgrid( double x0, double x1, double y0, double y1 );

   // Plot a function in the given color. We draw straight lines
   // between the points: 

   void plot( const std::function< double( double ) > & f, 
              double x0, double x1, scenery::color col );
}

#endif

