
#include "fourmatrix.h"
#include <cstdio> 

linalg::fourmatrix::fourmatrix( )
{
   for( unsigned int i = 0; i < 4; ++ i )
      for( unsigned int j = 0; j < 4; ++ j )
      {
         if( i == j )
            repr [i][j] = 1.0;
         else
            repr [i][j] = 0.0;
      }
}


linalg::fourmatrix::fourmatrix( const matrix& m )
{
   for( unsigned int i = 0; i < 3; ++ i )  
   {
      for( unsigned int j = 0; j < 3; ++ j )
         repr [i][j] = m. repr [i][j];
      repr [i][3] = 0.0;
   }

   for( unsigned int j = 0; j < 3; ++ j )
      repr [3][j] = 0.0;

   repr [3][3] = 1.0;
}


linalg::fourmatrix::fourmatrix( const linalg::vector& v )
{
   for( unsigned int i = 0; i < 4; ++ i )
      for( unsigned int j = 0; j < 3; ++ j )
      {
         if( i == j )
            repr [i][j] = 1.0;
         else
            repr [i][j] = 0.0;
      }

   repr [0][3] = v.x;
   repr [1][3] = v.y;
   repr [2][3] = v.z;
   repr [3][3] = 1.0;
}


linalg::fourmatrix::fourmatrix( const linalg::affinemap& map )
{
   for( unsigned int i = 0; i < 3; ++ i )
      for( unsigned int j = 0; j < 3; ++ j )
         repr [i][j] = map. m. repr [i][j];

   repr [0][3] = map.v.x;
   repr [1][3] = map.v.y;
   repr [2][3] = map.v.z;

   repr [3][0] = 0.0;
   repr [3][1] = 0.0;
   repr [3][2] = 0.0;
   repr [3][3] = 1.0;
}


linalg::fourmatrix linalg::fourmatrix::frustum( double x0, double x1,
                                                double y0, double y1,
                                                double z0, double z1 )
{
   if( x0 >= x1 )
      throw std::runtime_error( "frustum: x0 must be left of x1" );
   if( y0 >= y1 )
      throw std::runtime_error( "frustum: y0 must be below y1" );
   if( 0 <= z0 )
      throw std::runtime_error( "frustum: z0 must be in front of viewer" );
    if( z0 <= z1 )
      throw std::runtime_error( "frustum: z1 must be further away than z0" );

   fourmatrix result;

   result. repr [0][0] = 2.0 / ( x1 - x0 );
   result. repr [0][1] = 0.0;
   result. repr [0][2] = ( x0 + x1 ) / ( x1 - x0 );
   result. repr [0][3] = 0.0;

   result. repr [1][0] = 0.0;
   result. repr [1][1] = 2.0 / ( y1 - y0 );
   result. repr [1][2] = ( y0 + y1 ) / ( y1 - y0 );
   result. repr [1][3] = 0.0;

   result. repr [2][0] = 0.0;
   result. repr [2][1] = 0.0;
   result. repr [2][2] = ( z0 + z1 ) / ( z0 - z1 );
   result. repr [2][3] = 2.0 * z0 * z1 / ( z1 - z0 );

   result. repr [3][0] = 0.0;
   result. repr [3][1] = 0.0;
   result. repr [3][2] = -1.0;
   result. repr [3][3] = 0.0;

   return result; 
}

linalg::fourmatrix linalg::fourmatrix::orthographic( double x0, double x1,
                                                     double y0, double y1,
                                                     double z0, double z1 )
{
   if( x0 >= x1 )
      throw std::runtime_error( "ortho: x0 must be left of x1" );
   if( y0 >= y1 )
      throw std::runtime_error( "ortho: y0 must be below y1" );
   if( z0 <= z1 )
      throw std::runtime_error( "ortho: z1 must be further away than z0" );

   fourmatrix result;

   result. repr [0][0] = 2.0 / ( x1 - x0 );
   result. repr [0][1] = 0.0;
   result. repr [0][2] = 0.0;
   result. repr [0][3] = ( x0 + x1 ) / ( x0 - x1 ); 

   result. repr [1][0] = 0.0;
   result. repr [1][1] = 2.0 / ( y1 - y0 );
   result. repr [1][2] = 0.0;
   result. repr [1][3] = ( y0 + y1 ) / ( y0 - y1 );

   result. repr [2][0] = 0.0;
   result. repr [2][1] = 0.0;
   result. repr [2][2] = 2.0 / ( z1 - z0 ); 
   result. repr [2][3] = ( z0 + z1 ) / ( z0 - z1 );

   result. repr [3][0] = 0.0;
   result. repr [3][1] = 0.0;
   result. repr [3][2] = 0.0;
   result. repr [3][3] = 1.0;

   return result;
}

linalg::fourvector linalg::fourmatrix::apply( const fourvector& v ) const
{
   return fourvector(
   repr[0][0] * v.x + repr[0][1] * v.y + repr[0][2] * v.z + repr[0][3] * v.w,
   repr[1][0] * v.x + repr[1][1] * v.y + repr[1][2] * v.z + repr[1][3] * v.w,
   repr[2][0] * v.x + repr[2][1] * v.y + repr[2][2] * v.z + repr[2][3] * v.w,
   repr[3][0] * v.x + repr[3][1] * v.y + repr[3][2] * v.z + repr[3][3] * v.w);
}


linalg::fourmatrix linalg::operator * ( const fourmatrix& m1, 
                                        const fourmatrix& m2 )
{
   fourmatrix res;

   for( unsigned int i = 0; i < 4; ++ i )
      for( unsigned int j = 0; j < 4; ++ j )
      {
         res. repr [i][j] =
            m1. repr [i][0] * m2. repr [0][j] +
            m1. repr [i][1] * m2. repr [1][j] +
            m1. repr [i][2] * m2. repr [2][j] +
            m1. repr [i][3] * m2. repr [3][j]; 
      }

   return res; 
}


std::ostream& linalg::operator << ( std::ostream& stream,
                                    const fourmatrix& m )
{
   for( unsigned int i = 0; i < 4; ++ i )
   {
      if( i == 0 )
         stream << " /  ";
      if( i == 1 )
         stream << " |  ";
      if( i == 2 )
         stream << " |  ";
      if( i == 3 )
         stream << " \\  ";

      printf( "%.8lf     %.8lf     %.8lf     %.8lf", 
         m. repr [i][0], m. repr [i][1], m. repr [i][2], m. repr [i][3] );

      if( i == 0 )
         stream << "  \\\n";
      if( i == 1 )
         stream << "  |\n";
      if( i == 2 )
         stream << "  |\n";
      if( i == 3 ) 
         stream << "  /\n";
   }
   stream << "\n";
   return stream;
}

double* & linalg::operator << ( double* & p, const fourmatrix& m )
{
   for( unsigned int j = 0; j < 4; ++ j )
      for( unsigned int i = 0; i < 4; ++ i )
      {
         * ( p ++ ) = m. repr [i] [j];
      }
   return p;
}
 
