
#include "logger.h"
#include <cmath>


void logger::insert( double t, double y )
{
   if( values. size( ) == 0 )
   {
      values. push_back( interval( t, y ));
      return;
   }

   // If t is before the start of our history, we do nothing.

   if( t < values. front( ). t )
      return;

   size_t i = lrint( ( t - values. front( ). t ) / h );
      // lrint is C++-11 function which rounds to nearest long int. 

   while( i >= values. size( ))
      values. push_back( 
          interval( values. front( ). t + h * values. size( ), y ));

   values[i]. insert(y); 
}


void logger::erase( double t )
{
   while( values. size( ) && t > values. front( ). t )
      values. pop_front( );
}


double logger::operator( ) ( double t, int side ) const
{
   if( ! values. size( ))
      return 0.0;

   // We do this, because we assume that we are close:

   if( t < values. front( ). t )
      return side < 0 ? values. front( ). y0 : values. front( ). y1;

   size_t i = lrint( ( t - values. front( ). t ) / h );
   if( i >= values. size( ))
      i = values. size( ) - 1;  // We assume that we are close. 

   return side < 0 ? values[i]. y0 : values[i]. y1;
}


std::ostream& operator << ( std::ostream& out, const logger& l )
{
   out << "logger( " << l.h << " )\n";
   for( auto p = l. values. begin( ); p != l. values. end( ); ++ p )
   {
      out << "   " << p -> t << " ==> ";
      out << "[ " << p -> y0 << " ... " << p -> y1 << " ]\n";
   }
   return out;
}


