
#ifndef LOGGER_INCLUDED
#define LOGGER_INCLUDED 1

#include <deque> 
#include <iostream>

class logger
{

   struct interval
   {
      double t; 
      double y0;
      double y1;

      interval( double t, double y )
         : t{t}, y0{y}, y1{y}
      { }

      // If y is outside of interval [y0,y1], then extend the interval: 

      void insert( double y )
      {
         if( y < y0 ) 
            y0 = y;
         else
            if( y > y1 ) y1 = y;
      }
   };

   double h; 
   std::deque< interval > values;

public: 
   // h is the resolution.

   explicit logger( double h = 1.0E-3 )
      : h{h}
   { }
      
   void erase( double t );
      // Forget everything that was before t.

   // It is assumed that insertions are in increasing order of t.

   void insert( double t, double y ); 

   size_t size( ) const { return values. size( ); }

   double operator( ) ( double t, int side = -1 ) const; 
      // Returns either the lowest (side == -1) or the highest
      // value (side == 1), that was registered for t.
      // The user should plot the log twice, one time with side = -1, and
      // one time with side = 1.

   friend std::ostream& operator << ( std::ostream& , const logger& ); 
};

std::ostream& operator << ( std::ostream& , const logger& );

#endif

