
#include "half_atan.h"
#include <cmath>

// We have p^2 - q^2 = x, 
//         2pq = y.
//
// Solving gives the equation 4 p^4 - 4 x p^2 - y^2 = 0.

std::pair< double, double > math::half_atan( double x, double y )
{
   double xy = sqrt( x*x + y*y );

   double p = sqrt( 0.5 * (  x + xy ));
   double q = sqrt( 0.5 * ( -x + xy ));

   if( y < 0 )
      q = -q;  // Because pq = 0.5 sqrt( y^2 ), and we need
               // pq = 0.5 y. 

   return std::pair< double, double > ( p, q );
}
 
