
// Written by Hans de Nivelle, Jan 2017.

// A force_torque consists of a force f and a torque tq
// relative to the origine.

#ifndef PHYSICS_FORCE_TORQUE_INCLUDED 
#define PHYSICS_FORCE_TORQUE_INCLUDED    1

#include "../linalg/vector.h"

namespace physics
{

   struct force_torque
   {
      linalg::vector f;
      linalg::vector tq; 

      force_torque( ) 
         : f{0,0,0},  
           tq{0,0,0}
      { } 


      force_torque( const linalg::vector& f, const linalg::vector& tq )
         : f{f},
           tq{tq} 
      { }

   };


   inline force_torque operator - ( force_torque ft )
   {
      return { -ft.f, -ft. tq };
   }

   inline force_torque operator + ( force_torque ft1, force_torque ft2 )
   {
      return { ft1.f + ft2.f, ft1.tq + ft2.tq }; 
   }

   inline force_torque operator - ( force_torque ft1, force_torque ft2 )
   {
      return { ft1.f - ft2.f, ft1.tq - ft2.tq }; 
   }

   inline void operator += ( force_torque& ft1, force_torque ft2 )
   {   
      ft1.f += ft2.f; ft1.tq += ft2.tq;
   }

   inline void operator -= ( force_torque& ft1, force_torque ft2 )
   {
      ft1.f -= ft2.f; ft1.tq -= ft2.tq;  
   }


   
   inline force_torque operator * ( force_torque ft, double d )
   {
      return { ft.f * d, ft.tq * d }; 
   }

   inline force_torque operator * ( double d, force_torque ft )
   {
      return { d * ft.f, d * ft.tq };
   }

   inline force_torque operator / ( force_torque ft, double d )
   {
      return { ft.f / d, ft.tq / d }; 
   }

   inline void operator *= ( force_torque& ft, double d )
   {
      ft.f *= d; ft.tq *= d; 
   }

   inline void operator /= ( force_torque& ft, double d )
   {
      ft.f /= d; ft.tq /= d; 
   }


   inline 
   force_torque force_point( linalg::vector force,
                             linalg::vector acting_point )
      { return { force, linalg::crossproduct( acting_point, force ) }; }


   inline 
   std::ostream& operator << ( std::ostream& stream, const force_torque ft )
   {
      stream << "force_torque( " << ft. f << ", " << ft. tq << " )";
      return stream;
   }

}

#endif 

