
#ifndef TIMER_INCLUDED
#define TIMER_INCLUDED  1

#include <chrono>

class timer
{
   using clock = std::chrono::steady_clock;

   clock::time_point creation;   // Time on which the timer was created,
                                 // in system time. 
   double t0;                    // Time on which the timer was created
                                 // in user time.  
   double scale;                 // Time scale, should be > 0.0.
                                 // scale = 10 means that 
                                 // our simulation runs 10 times
                                 // faster than the system clock. 

public: 
   timer( double t0, double scale = 1.0 )
      : creation{ clock::now( ) },
        t0{ t0 },
        scale{ scale } 
   { }


   double gettime( ) const
   {
      std::chrono::duration< double > d = clock::now( ) - creation;
      return t0 + d. count( ) * scale;  
   }

   double getscale( ) const { return scale; }

   // Just assigning to scale would mess up the time calculations:

   void setscale( double newscale )
   {
      double t = gettime( );
      creation = clock::now( );
      t0 = t;
      scale = newscale;
   }
};

#endif

